/*
 * (C) Copyright 2007-2011
 * Allwinner Technology Co., Ltd. <www.allwinnertech.com>
 * Tom Cubie <tangliang@allwinnertech.com>
 *
 * Boot an image which is generated by android mkbootimg tool
 *
 * (C) Copyright 2000-2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Misc boot support
 */
#include <common.h>
#include <command.h>
#include <net.h>

#ifdef CONFIG_CMD_BOOTA
#include <fastboot.h>

static unsigned char boot_hdr[512];

extern int do_boota_linux (struct fastboot_boot_img_hdr *hdr);

int do_boota (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong	addr;
	unsigned kaddr, raddr;

	if (argc < 2)
		return cmd_usage(cmdtp);

	tick_printf(__FILE__, __LINE__);

	addr = simple_strtoul(argv[1], NULL, 16);

	struct fastboot_boot_img_hdr *fb_hdr = (struct fastboot_boot_img_hdr *)addr;
	image_header_t *hdr =(image_header_t *)(addr + CFG_FASTBOOT_MKBOOTIMAGE_PAGE_SIZE);
#ifdef DEBUG
	printf("---------------------\n");
	printf("kernel size: 0x%x \n", fb_hdr->kernel_size);
	printf("kernel addr: 0x%x \n", fb_hdr->kernel_addr);

	printf("ramdisk size: 0x%x \n", fb_hdr->ramdisk_size);
	printf("ramdisk addr: 0x%x \n", fb_hdr->ramdisk_addr);

	printf("second size: 0x%x \n", fb_hdr->second_size);
	printf("second addr: 0x%x \n", fb_hdr->second_addr);

	printf("name: %s\n", fb_hdr->name);
	printf("cmdline: %s\n", fb_hdr->cmdline);
#endif
	/* save the image header somewhere */
	memcpy(boot_hdr, (void*) addr, sizeof(*hdr));
	if (memcmp(fb_hdr->magic, FASTBOOT_BOOT_MAGIC, 8)) {
		puts("boota: bad boot image magic, maybe not a boot.img?\n");
		return 1;
	}

	kaddr = addr + fb_hdr->page_size;
	raddr = kaddr + ALIGN(fb_hdr->kernel_size, fb_hdr->page_size);
#ifdef DEBUG
	printf("moving kernel from %x to: %x, size 0x%x\n", kaddr, fb_hdr->kernel_addr, fb_hdr->kernel_size);
	printf("moving ramdisk from %x to: %x, size 0x%x\n", raddr, fb_hdr->ramdisk_addr, fb_hdr->ramdisk_size);
#endif
    tick_printf(__FILE__, __LINE__);
    if((fb_hdr->unused[0] == 0x55) &&(fb_hdr->unused[1] == 0xaa))
    {
        #ifdef DEBUG
        printf("bootimg load in boot1!\n");
        #endif
        do_boota_linux(fb_hdr);
        tick_printf(__FILE__, __LINE__);
    }else
    {
        #ifdef DEBUG
        printf("memmove bootimg!\n");
        #endif
        memmove((void*) fb_hdr->kernel_addr, (const void *)kaddr, fb_hdr->kernel_size);
        tick_printf(__FILE__, __LINE__);
    	memmove((void*) fb_hdr->ramdisk_addr, (const void *)raddr, fb_hdr->ramdisk_size);
    	tick_printf(__FILE__, __LINE__);
    	do_boota_linux(fb_hdr);
    }


	puts("Boot linux failed, control return to monitor\n");

	return 0;
}

U_BOOT_CMD(
	boota,	3,	1,	do_boota,
	"boota   - boot android bootimg from memory\n",
	"<addr>\n    - boot application image stored in memory\n"
	"\t'addr' should be the address of boot image which is kernel+ramdisk.img\n"
);
#endif
